
__name__ = 'ezaggrid'
name_url = __name__.replace('_', '-')

__packages__ = [__name__]
__version__ = '0.1.1'
__description__ = 'display dataframe in ag-grid'
__long_description__ = 'See repo README'
__author__ = 'oscar6echo'
__author_email__ = 'olivier.borderies@gmail.com'
__url__ = 'https://github.com/{}/{}'.format(__author__,
                                            name_url)
__download_url__ = 'https://github.com/{}/{}/tarball/{}'.format(__author__,
                                                                name_url,
                                                                __version__)
__keywords__ = ['javascript', 'dataframe', 'ag-grid', 'display']
__license__ = 'MIT'
__classifiers__ = ['Development Status :: 4 - Beta',
                   'License :: OSI Approved :: MIT License',
                   'Programming Language :: Python :: 3.5',
                   'Programming Language :: Python :: 3.6'
                   ]
__include_package_data__ = True
__package_data__ = {
    'templates':
        ['templates/iframe.tpl.html',
         'templates/main.tpl.html',
         'templates/script.tpl.js'
         'templates/json.js'
         'templates/helpers.js'
        ],
}

__zip_safe__ = False




import os

from IPython.display import display, HTML

from .params import Params
from .template import Template


class AgGrid:
    """
    """

    def __init__(self,
                 width=None,
                 height=None,
                 theme=None,
                 css_rules=None,
                 quick_filter=None,
                 export_csv=None,
                 export_excel=None,
                 std_types=None,
                 grid_data=None,
                 grid_options=None,
                 license=None,
                 iframe=False,
                 verbose=False):

        dic = {'width': width,
               'height': height,
               'theme': theme,
               'css_rules': css_rules,
               'quick_filter': quick_filter,
               'export_csv': export_csv,
               'export_excel': export_excel,
               'std_types': std_types,
               'grid_data': grid_data,
               'grid_options': grid_options,
               'license': license,
               'verbose': verbose
               }
        dic = { k: v for k, v in dic.items() if v is not None}

        self.params = Params(**dic)

        self.template = Template(params=self.params,
                                 iframe=iframe)

        self.html = self.template.content

    def show(self):
        """
        display image tabs
        """
        display(HTML(self.html))




import random
import simplejson as json

import pandas as pd

from copy import deepcopy as copy

from .util import json_serial, sanitize_str, sanitize_struct, \
                  build_css_rules, encode_b64, \
                  update_columnDefs, update_columnTypes


class Params:
    """
    """

    def __init__(self,
                 width=700,
                 height=300,
                 theme='ag-theme-fresh',
                 css_rules=None,
                 quick_filter=False,
                 export_csv=False,
                 export_excel=False,
                 std_types=True,
                 grid_data=None,
                 grid_options=None,
                 license=None,
                 verbose=False,
                 **kwargs):
        """
        Set params incl. defaults
        """
        # attributes
        self.uuid = random.randint(0, int(1e9))
        self.width = width
        self.height = height + 20
        self.theme = theme
        self.css_rules = css_rules
        self.quick_filter = quick_filter
        self.export_csv = export_csv
        self.export_excel = export_excel
        self.std_types = std_types
        self.grid_data = grid_data
        self.grid_options = grid_options
        self.license = license

        # if not self.widthIframe:
        #     self.widthIframe = self.width + 2 * self.borderPx
        # if not self.heightIframe:
        #     isTitle = self.titleText is not None
        #     self.heightIframe = self.height + 30 * isTitle + 50 + 2 * self.borderPx + 5

        if self.std_types:
            self.grid_options = update_columnDefs(self.grid_data, self.grid_options)
            self.grid_options = update_columnTypes(self.grid_options)

        self.valid = self.check(verbose=verbose)
        # print('grid_data')
        self.grid_data_json = self.build(self.grid_data)
        # print('grid_options')
        # print(grid_options)
        self.grid_options_json = self.build(self.grid_options, sanitize=True)
        # self.dic_grid_data = self.build(self.grid_data)
        # self.dic_grid_options = self.build(self.grid_options)

        if self.css_rules is not None:
            self.css_rules = build_css_rules(self.css_rules)

        if self.license is not None:
            self.license = encode_b64(self.license)

    def check(self, verbose=False):
        """
        """
        msg = 'width must be a number of pixels'
        assert isinstance(self.width, int), msg

        msg = 'height must be a number of pixels'
        assert isinstance(self.height, int), msg

        msg = 'quick_filter must be a boolean'
        assert isinstance(self.quick_filter, bool), msg

        msg = 'export_csv must be a boolean'
        assert isinstance(self.export_csv, bool), msg

        msg = 'exportExport must be a boolean'
        assert isinstance(self.export_excel, bool), msg



        if self.css_rules is not None:
            msg = 'css_rules must be a string'
            assert isinstance(self.css_rules, str), msg

        li_theme = ['ag-theme-fresh',
                    'ag-theme-dark',
                    'ag-theme-blue',
                    'ag-theme-material',
                    'ag-theme-bootstrap']
        msg = 'theme must be one of {}'.format(li_theme)
        assert self.theme in li_theme, msg

        msg = 'grid_data must be a dict or a dataframe'
        assert isinstance(self.grid_data, (list, pd.core.frame.DataFrame)), msg
        if isinstance(self.grid_data, list):
            for e in self.grid_data:
                msg = 'each element of grid_data must be a dict'
                assert isinstance(e, dict), msg

        if verbose:
            print('data is valid')

        return True

    def build(self, data, sanitize=False):
        """
        """
        if data is None:
            return None

        if isinstance(data, pd.core.frame.DataFrame):
            data2 = data.to_dict(orient='records')
        else:
            data2 = data

        if sanitize:
            data2 = sanitize_struct(data2)
            # print(data2)

        data_json = json.dumps(data2, default=json_serial, ignore_nan=True)
        return data_json

    def to_dict(self):
        """
        """
        d = copy(self.__dict__)
        d = {k: v for k, v in d.items() if v is not None}
        return d

    def pprint(self, indent=2):
        """
        """
        d = json.dumps(self.to_dict(),
                       sort_keys=True,
                       indent=indent)
        print(d)

    def __repr__(self):
        return str(self.to_dict())




import os
import re
import collections
import base64

import datetime as dt
import pandas as pd


def json_serial(obj):
    """
    """
    if isinstance(obj, (dt.datetime, dt.date, pd.Timestamp)):
        return obj.isoformat()

    return obj


# def strip_comments(code):
#     if code.startswith('function'):
#         print(code)
#     lines = code.split('\n')
#     if code.startswith('function'):
#         print(lines)
#     lines = [e.strip() for e in lines]
#     if code.startswith('function'):
#         print(lines)
#     lines = [e for e in lines if not e.startswith('//')]
#     if code.startswith('function'):
#         print(lines)
#     code = '\n'.join(lines)
#     if code.startswith('function'):
#         print(code)

#     return code


def strip_comments(code):
    lines = code.split('\n')
    lines = [e.strip() for e in lines]
    lines = [e for e in lines if not e.startswith('//')]
    code = '\n'.join(lines)
    return code


def sanitize_str(string):
    string2 = strip_comments(string)
    string2 = string2.replace('\n', '')
    string2 = string2.replace('\t', ' ')
    string2 = string2.replace('\"', '\'')
    return string2


def sanitize_struct(e):
    #     print('\n', type(e), '\n', e)
    if isinstance(e, list):
        return [sanitize_struct(sub_e) for sub_e in e]
    elif isinstance(e, dict):
        return {k: sanitize_struct(v) for k, v in e.items()}
    elif isinstance(e, str):
        return sanitize_str(e)
    else:
        return e


def update_columnDefs(df, grid_options):
    colDefs = grid_options.get('columnDefs', [])
    li_field = []
    for colDef in colDefs:
        field = colDef.get('field')
        li_field.append(field)
        if field:
            if field in df.columns:
                dic = colDef
                col = df[field]
                if col.dtype.kind in 'O':
                    # string
                    dic['type'] = 'textColumn'
                if col.dtype.kind in 'ifc':
                    # number
                    dic['type'] = 'numberColumn'
                if col.dtype.kind in 'M':
                    # date
                    dic['type'] = 'dateColumn'
    for c in df.columns:
        if c not in li_field:
            dic = {}
            col = df[c]
            if col.dtype.kind in 'O':
                # string
                dic['type'] = 'textColumn'
            if col.dtype.kind in 'ifc':
                # number
                dic['type'] = 'numberColumn'
            if col.dtype.kind in 'M':
                # date
                dic['type'] = 'dateColumn'
            colDefs.append(dic)
    grid_options['columnDefs'] = colDefs
    return grid_options


def update_columnTypes(grid_options):
    columnTypes = grid_options.get('columnTypes', {})

    numberColumn = {
        'filter': 'agNumberColumnFilter'
    }
    if not 'numberColumn' in columnTypes:
        columnTypes['numberColumn'] = numberColumn

    textColumn = {
        'filter': 'agTextColumnFilter'
    }
    if not 'textColumn' in columnTypes:
        columnTypes['textColumn'] = textColumn

    dateColumn = {
        'valueFormatter': 'helpers.dateFormatter',
        'filter': 'agDateColumnFilter',
        'filterParams': {
            'comparator': 'helpers.compareDates'
        }
    }
    if not 'dateColumn' in columnTypes:
        columnTypes['dateColumn'] = dateColumn

    grid_options['columnTypes'] = columnTypes
    return grid_options


def build_css_rules(css_rules):
    css_rules = re.findall(r'[^\{]+\{[^\}]*\}',
                           css_rules,
                           re.MULTILINE)
    css_rules = [sanitize_str(e) for e in css_rules]
    return css_rules


def get_license(filename='.ag_grid_license'):
    path = os.path.join(os.path.expanduser('~'), filename)
    with open(path, 'r') as f:
        license = f.read()
    return license

def encode_b64(string):
    return base64.b64encode(bytes(string, 'utf-8')).decode('utf-8')

